//
//  Incident.cpp
//
//

#include <math.h>

#include "resource.h"
#ifdef __WXMSW__
	#include "word.h"
#else
	#include "word_e.h"
#endif
#include "incident.h"
#include "wx/wx.h"
#include "wx/string.h"
#include "KidsCindyCanvas.h"
#include "KidsCindyFrame.h"


bool MyCanvas::point_coincide_local(GPoint* pt1,GPoint* pt2)
{
	if(pt1==pt2){
		return false;
	}
	double den=sqrt(norm2(pt1->xx)+norm2(pt1->yy)+norm2(pt1->zz))*sqrt(norm2(pt2->xx)+norm2(pt2->yy)+norm2(pt2->zz));
	CComplex num=pt1->xx * conjugate(pt2->xx) + pt1->yy * conjugate(pt2->yy) + pt1->zz * conjugate(pt2->zz);
	double costh2= norm2(num/CComplex(den,0.0));
	if(costh2 > 0.99999){
		return true;
	}
	return false;
}

bool MyCanvas::point_coincide_global(GConst* cnp,GPoint* pt1,GPoint* pt2)
{
	if(cnp==NULL){
		return true;
	}
	if(pt1==pt2){
		return false;
	}
	if(!point_coincide_local(pt1,pt2)){
		return false;
	}
	if(cnp->Const_Type==CONST_FREE){
		// push the coordinate
		CComplex pushxx=cnp->PInput1->xx;
		CComplex pushyy=cnp->PInput1->yy;
		CComplex pushzz=cnp->PInput1->zz;
		// purterb once and evaluate
		cnp->PInput1->xx += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!point_coincide_local(pt1,pt2)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// purterb again and evaluate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!point_coincide_local(pt1,pt2)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// pop the coordinate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy = pushyy;
		cnp->PInput1->zz = pushzz;
		EvaluateConst(cnp);
		// go to next step
		return point_coincide_global(cnp->next,pt1,pt2);
	}
	return point_coincide_global(cnp->next,pt1,pt2);
}

bool MyCanvas::line_coincide_local(GLine* ln1,GLine* ln2)
{
	if(ln1==ln2){
		return false;
	}
	double den=sqrt(norm2(ln1->xx)+norm2(ln1->yy)+norm2(ln1->zz))*sqrt(norm2(ln2->xx)+norm2(ln2->yy)+norm2(ln2->zz));
	CComplex num=ln1->xx * conjugate(ln2->xx) + ln1->yy * conjugate(ln2->yy) + ln1->zz * conjugate(ln2->zz);
	double costh2= norm2(num/CComplex(den,0.0));
	if(costh2 > 0.99999){
		return true;
	}
	return false;
}

bool MyCanvas::line_coincide_global(GConst* cnp,GLine* ln1,GLine* ln2)
{
	if(cnp==NULL){
		return true;
	}
	if(ln1==ln2){
		return false;
	}
	if(!line_coincide_local(ln1,ln2)){
		return false;
	}
	if(cnp->Const_Type==CONST_FREE){
		// push the coordinate
		CComplex pushxx=cnp->PInput1->xx;
		CComplex pushyy=cnp->PInput1->yy;
		CComplex pushzz=cnp->PInput1->zz;
		// purterb once and evaluate
		cnp->PInput1->xx += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!line_coincide_local(ln1,ln2)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// purterb again and evaluate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!line_coincide_local(ln1,ln2)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// pop the coordinate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy = pushyy;
		cnp->PInput1->zz = pushzz;
		EvaluateConst(cnp);
		// go to next step
		return line_coincide_global(cnp->next,ln1,ln2);
	}
	return line_coincide_global(cnp->next,ln1,ln2);
}

bool MyCanvas::circle_coincide_local(GCircle* ci1,GCircle* ci2)
{
	if(ci1==ci2){
		return false;
	}
	double den=norm2(ci1->xx - ci2->xx)+norm2(ci1->yy - ci2->yy)+norm2(ci1->rr - ci2->rr);
	if(den <0.00001){
		return true;
	}
	return false;
}

bool MyCanvas::circle_coincide_global(GConst* cnp,GCircle* ci1,GCircle* ci2)
{
	if(cnp==NULL){
		return true;
	}
	if(ci1==ci2){
		return false;
	}
	if(!circle_coincide_local(ci1,ci2)){
		return false;
	}
	if(cnp->Const_Type==CONST_FREE){
		// push the coordinate
		CComplex pushxx=cnp->PInput1->xx;
		CComplex pushyy=cnp->PInput1->yy;
		CComplex pushzz=cnp->PInput1->zz;
		// purterb once and evaluate
		cnp->PInput1->xx += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!circle_coincide_local(ci1,ci2)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// purterb again and evaluate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!circle_coincide_local(ci1,ci2)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// pop the coordinate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy = pushyy;
		cnp->PInput1->zz = pushzz;
		EvaluateConst(cnp);
		// go to next step
		return circle_coincide_global(cnp->next,ci1,ci2);
	}
	return circle_coincide_global(cnp->next,ci1,ci2);
}

bool MyCanvas::incident_LP_local(GLine* ln, GPoint* pt)
{
	CComplex dist=ln->xx * pt->xx + ln->yy * pt->yy + ln->zz * pt->zz;
	double distdenom=ln->xx.real()*ln->xx.real()+ln->yy.real()*ln->yy.real();
	if((dist.re * dist.re + dist.im * dist.im)/distdenom < 0.00001){
		return true;
	}
	return false;
}

bool MyCanvas::incident_LP_global(GConst* cnp,GLine* ln, GPoint* pt)
{
	if(cnp==NULL){
		return false;
	}
	if(pt->invisible || ln->invisible){
		return false;
	}
	if(cnp->Const_Type==CONST_FREE){
		// push the coordinate
		CComplex pushxx=cnp->PInput1->xx;
		CComplex pushyy=cnp->PInput1->yy;
		CComplex pushzz=cnp->PInput1->zz;
		// purterb once and evaluate
		cnp->PInput1->xx += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!incident_LP_local(ln,pt)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// purterb again and evaluate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!incident_LP_local(ln,pt)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// pop the coordinate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy = pushyy;
		cnp->PInput1->zz = pushzz;
		EvaluateConst(cnp);
		// go to next step
		return incident_LP_global(cnp->next,ln,pt);
	}
	return incident_LP_global(cnp->next,ln,pt);
}

bool MyCanvas::incident_CP_local(GCircle* ci, GPoint* pt)
{
	double dist=norm2(ci->xx - pt->xz())+norm2(ci->yy - pt->yz());
	if(dist >= ci->rr.real()* 0.99999 && dist <= ci->rr.real()* 1.00001){
		return true;
	}
	return false;
}

bool MyCanvas::incident_CP_global(GConst* cnp,GCircle* ci, GPoint* pt)
{
	if(cnp==NULL){
		return false;
	}
	if(pt->invisible || ci->invisible){
		return false;
	}
	if(cnp->Const_Type==CONST_FREE){
		// push the coordinate
		CComplex pushxx=cnp->PInput1->xx;
		CComplex pushyy=cnp->PInput1->yy;
		CComplex pushzz=cnp->PInput1->zz;
		// purterb once and evaluate
		cnp->PInput1->xx += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!incident_CP_local(ci,pt)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// purterb again and evaluate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!incident_CP_local(ci,pt)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// pop the coordinate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy = pushyy;
		cnp->PInput1->zz = pushzz;
		EvaluateConst(cnp);
		// go to next step
		return incident_CP_global(cnp->next,ci,pt);
	}
	return incident_CP_global(cnp->next,ci,pt);
}

bool MyCanvas::tangent_CL_local(GCircle* ci, GLine* ln)
{
	double distnum = (ci->xx*ln->xx + ci->yy*ln->yy + ln->zz).real();
	double distnum2 = distnum*distnum;
	double distden2 = (ln->xx*ln->xx + ln->yy*ln->yy).real();
	if(distnum2/distden2 >= ci->rr.real()* 0.99999 && distnum2/distden2 <= ci->rr.real()* 1.00001){
		return true;
	}
	return false;
}

bool MyCanvas::tangent_CL_global(GConst* cnp,GCircle* ci, GLine* ln)
{
	if(cnp==NULL){
		return false;
	}
	if(ci->invisible || ln->invisible){
		return false;
	}
	if(cnp->Const_Type==CONST_FREE){
		// push the coordinate
		CComplex pushxx=cnp->PInput1->xx;
		CComplex pushyy=cnp->PInput1->yy;
		CComplex pushzz=cnp->PInput1->zz;
		// purterb once and evaluate
		cnp->PInput1->xx += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!tangent_CL_local(ci,ln)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// purterb again and evaluate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!tangent_CL_local(ci,ln)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// pop the coordinate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy = pushyy;
		cnp->PInput1->zz = pushzz;
		EvaluateConst(cnp);
		// go to next step
		return tangent_CL_global(cnp->next,ci,ln);
	}
	return tangent_CL_global(cnp->next,ci,ln);
}

bool MyCanvas::tangent_CC_local(GCircle* ci1, GCircle* ci2)
{
	double dist1 = sqrt(ci1->rr.real());
	double dist2 = sqrt(ci2->rr.real());
	double dist3 = sqrt(((ci1->xx-ci2->xx)*(ci1->xx-ci2->xx)+ (ci1->yy - ci2->yy)*(ci1->yy - ci2->yy)).real());
	if(dist1+dist2 >= dist3 * 0.9999 && dist1+dist2 <= dist3 * 1.0001){
		return true;
	}
	if(dist2+dist3 >= dist1 * 0.9999 && dist2+dist3 <= dist1 * 1.0001){
		return true;
	}
	if(dist3+dist1 >= dist2 * 0.9999 && dist3+dist1 <= dist2 * 1.0001){
		return true;
	}
	return false;
}

bool MyCanvas::tangent_CC_global(GConst* cnp,GCircle* ci1, GCircle* ci2)
{
	if(cnp==NULL){
		return true;
	}
	if(ci1->invisible || ci2->invisible){
		return false;
	}
	if(cnp->Const_Type==CONST_FREE){
		// push the coordinate
		CComplex pushxx=cnp->PInput1->xx;
		CComplex pushyy=cnp->PInput1->yy;
		CComplex pushzz=cnp->PInput1->zz;
		// purterb once and evaluate
		cnp->PInput1->xx += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!tangent_CC_local(ci1,ci2)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// purterb again and evaluate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy += (CComplex(0.1,0.0) * cnp->PInput1->zz); 
		EvaluateConst(cnp);
		if(!tangent_CC_local(ci1,ci2)){
			cnp->PInput1->xx = pushxx;
			cnp->PInput1->yy = pushyy;
			cnp->PInput1->zz = pushzz;
			EvaluateConst(cnp);
			return false;
		}
		// pop the coordinate
		cnp->PInput1->xx = pushxx;
		cnp->PInput1->yy = pushyy;
		cnp->PInput1->zz = pushzz;
		EvaluateConst(cnp);
		// go to next step
		return tangent_CC_global(cnp->next,ci1,ci2);
	}
	return tangent_CC_global(cnp->next,ci1,ci2);
}

void MyCanvas::AssignIncident(GLine*ln,GPoint*pt,double icpara)
{
	for(int i=0;i<NUM_OF_BUFFER;i++){
		if(incidentbuffer[i]._used==false){
			//printf("Debug:AssignIncident:Line %s, Point %s, value %f\n",ln->_name.c_str(),pt->_name.c_str(),icpara);
			incidentbuffer[i]._used=true;
			*(ln->last_ic)=&(incidentbuffer[i]);
			(*(ln->last_ic))->_icpt=pt;
			(*(ln->last_ic))->_icnext=NULL;
			(*(ln->last_ic))->_icpara=CComplex(icpara,0.0);
			ln->last_ic=&((*(ln->last_ic))->_icnext);
			ln->NOics ++;
			return;
		}
	}
}

void MyCanvas::AssignIncident(GCircle*ci,GPoint*pt,double icpara)
{
	for(int i=0;i<NUM_OF_BUFFER;i++){
		if(incidentbuffer[i]._used==false){
			incidentbuffer[i]._used=true;
			*(ci->last_ic)=&(incidentbuffer[i]);
			(*(ci->last_ic))->_icpt=pt;
			(*(ci->last_ic))->_icnext=NULL;
			(*(ci->last_ic))->_icpara=CComplex(icpara,0.0);
			ci->last_ic=&((*(ci->last_ic))->_icnext);
			ci->NOics ++;
			return;
		}
	}
}

void MyCanvas::CheckIncidency(GPoint*pt,bool msg)
{
	// check incidency of the point and something
	wxString str1,str2,str3;
	char buff[1024];
	char buff2[1024];
	for(GLine* lnp=lns;lnp!=NULL;lnp=lnp->next){
		if(incident_LP_global(cns,lnp,pt)){
			bool newic=true;
			for(GIncident* icp=lnp->ics;icp!=NULL && newic;icp=icp->_icnext){
				if(icp->_icpt==pt){
					newic=false;
				}
			}
			if(newic){
				AssignIncident(lnp,pt);
				ResetLineIcPara(lnp);
				if(msg){
					wxString message;
					if(lnp->NOics>=3){
						sprintf(buff,"%s",(const char*)IncidentMsgPLAB.mb_str(wxConvUTF8));
						sprintf(buff2,buff,
								(char*)pt->_name.c_str(),
								(char*)lnp->ics->_icpt->_name.c_str(),
								(char*)lnp->ics->_icnext->_icpt->_name.c_str());
						message=wxString((char*)buff2,*wxConvCurrent);
					}
					else {
						sprintf(buff,"%s",(const char*)IncidentMsgPL.mb_str(wxConvUTF8));
						sprintf(buff2,buff,(char*)pt->_name.c_str());
						message=wxString((char*)buff2,*wxConvCurrent);
					}
					wxMessageBox(message,wxT("Incidency Test"),wxOK | wxICON_INFORMATION, this);
				}
			}
		}
	}
	// check incidency of the point and circles
	for(GCircle* cip=cis;cip!=NULL;cip=cip->next){
		if(incident_CP_global(cns,cip,pt)){
			bool newic=true;
			for(GIncident* icp=cip->ics;icp!=NULL && newic;icp=icp->_icnext){
				if(icp->_icpt==pt){
					newic=false;
				}
			}
			if(newic){
				AssignIncident(cip,pt);
				ResetCircleIcPara(cip);
				if(msg){
					wxString message;
					sprintf(buff,"%s",(const char*)IncidentMsgPC.mb_str(wxConvUTF8));
					sprintf(buff2,buff,(char*)pt->_name.c_str());
					message=wxString((char*)buff2,*wxConvCurrent);
					wxMessageBox(message,wxT("Incidency Test"),wxOK | wxICON_INFORMATION, this);
				}
			}
		}
	}
}

void MyCanvas::CheckIncidency(GLine*ln,bool msg)
{
	// check incidency of lines
	GConst* cnp;
	GPoint* ptp;
	char buff[1024],buff2[1024];
	for(ptp=pts;ptp!=NULL;ptp=ptp->next){
		if(incident_LP_global(cns,ln,ptp)){
			bool newic=true;
			for(GIncident* icp=ln->ics;icp!=NULL && newic;icp=icp->_icnext){
				if(icp->_icpt==ptp){
					newic=false;
				}
			}
			if(newic){
				AssignIncident(ln,ptp);
				ResetLineIcPara(ln);
				if(msg){
					wxString message;
					sprintf(buff,"%s",(const char*)IncidentMsg.mb_str(wxConvUTF8));
					sprintf(buff2,buff,(char*)ptp->_name.c_str());
					message=wxString((char*)buff2,*wxConvCurrent);
					wxMessageBox(message,wxT("Incidency Test"),wxOK | wxICON_INFORMATION, this);
				}
			}
		}
	}
	for(GCircle* cip=cis;cip!=NULL;cip=cip->next){
		if(tangent_CL_global(cns,cip,ln)){
			if(msg){
				for(cnp=cns;cnp!=NULL;cnp=cnp->next){
					if(cnp->COutput==cip && cnp->Const_Type==CONST_MULTI_CIRCLE){
						ptp=cnp->PInput1;
					}
					else if(cnp->COutput==cip && cnp->Const_Type==CONST_COMPASS){
						ptp=cnp->PInput3;
					}
				}
				wxString message;
				if(ln->NOics>=2){
					sprintf(buff,"%s",(const char*)TangentMsgLCAB.mb_str(wxConvUTF8));
					sprintf(buff2,buff,
						(char*)ln->ics->_icpt->_name.c_str(),
						(char*)ln->ics->_icnext->_icpt->_name.c_str(),
						(char*)ptp->_name.c_str());
					message=wxString((char*)buff2,*wxConvCurrent);
				}
				else {
					sprintf(buff,"%s",(const char*)TangentMsgLC.mb_str(wxConvUTF8));
					sprintf(buff2,buff,
							(char*)ptp->_name.c_str());
					message=wxString((char*)buff2,*wxConvCurrent);
				}
				wxMessageBox(message,wxT("Tangency Test"),wxOK | wxICON_INFORMATION, this);
			}
		}
	}
}

void MyCanvas::CheckIncidency(GCircle*ci,bool msg)
{
	// check incidency of the circle anmd points
	GPoint *ptp,*ptq=NULL;
	GConst *cnp;
	char buff[1024],buff2[1024];
	for(ptp=pts;ptp!=NULL;ptp=ptp->next){
		if(incident_CP_global(cns,ci,ptp)){
			bool newic=true;
			for(GIncident* icp=ci->ics;icp!=NULL && newic;icp=icp->_icnext){
				if(icp->_icpt==ptp){
					newic=false;
				}
			}
			if(newic){
				AssignIncident(ci,ptp);
				ResetCircleIcPara(ci);
				if(msg){
					wxString message;
					sprintf(buff,"%s",(const char*)IncidentMsgC.mb_str(wxConvUTF8));
					sprintf(buff2,buff,
							(char*)ptp->_name.c_str());
					message=wxString((char*)buff2,*wxConvCurrent);
					wxMessageBox(message,wxT("Incidency Test"),wxOK | wxICON_INFORMATION, this);
				}
			}
		}
	}
	for(GLine* lnp=lns;lnp!=NULL;lnp=lnp->next){
		if(tangent_CL_global(cns,ci,lnp)){
			if(msg){
				for(cnp=cns;cnp!=NULL;cnp=cnp->next){
					if(cnp->COutput==ci && cnp->Const_Type==CONST_MULTI_CIRCLE){
						ptp=cnp->PInput1;
					}
					else if(cnp->COutput==ci && cnp->Const_Type==CONST_COMPASS){
						ptp=cnp->PInput3;
					}
				}
				wxString message;
				if(lnp->NOics>=2){
					sprintf(buff,"%s",(const char*)TangentMsgCLAB.mb_str(wxConvUTF8));
					sprintf(buff2,buff,
							(char*)ptp->_name.c_str(),
							(char*)lnp->ics->_icpt->_name.c_str(),
							(char*)lnp->ics->_icnext->_icpt->_name.c_str());
					message=wxString((char*)buff2,*wxConvCurrent);
				}
				else {
					sprintf(buff,"%s",(const char*)TangentMsgCL.mb_str(wxConvUTF8));
					sprintf(buff2,buff,
							(char*)ptp->_name.c_str());
					message=wxString((char*)buff2,*wxConvCurrent);
				}
				wxMessageBox(message,wxT("Tangency Test"),wxOK | wxICON_INFORMATION, this);
			}
		}
	}
	for(GCircle* cip=cis;cip!=NULL;cip=cip->next){
		if(ci!=cip && tangent_CC_global(cns,ci,cip)){
			if(msg){
				for(cnp=cns;cnp!=NULL;cnp=cnp->next){
					if(cnp->COutput==ci && cnp->Const_Type==CONST_MULTI_CIRCLE){
						ptp=cnp->PInput1;
					}
					else if(cnp->COutput==ci && cnp->Const_Type==CONST_COMPASS){
						ptp=cnp->PInput3;
					}
					if(cnp->COutput==cip && cnp->Const_Type==CONST_MULTI_CIRCLE){
						ptq=cnp->PInput1;
					}
					else if(cnp->COutput==cip && cnp->Const_Type==CONST_COMPASS){
						ptq=cnp->PInput3;
					}
				}
				wxString message;
				sprintf(buff,"%s",(const char*)TangentMsgCC.mb_str(wxConvUTF8));
				sprintf(buff2,buff,
					(char*)ptp->_name.c_str(),
					(char*)ptq->_name.c_str());
				message=wxString((char*)buff2,*wxConvCurrent);
				wxMessageBox(message,wxT("Tangency Test"),wxOK | wxICON_INFORMATION, this);
			}
		}
	}
}

bool MyCanvas::CheckCoincidency(GPoint*pt)
{
	bool point_coincident=false;
	GPoint*pt1,*pt2=0;
	for(pt1=pts;pt1!=NULL && point_coincident==false;pt1=pt1->next){
		point_coincident=point_coincide_global(cns,pt1,pt);
		pt2=pt1;
	}
	if(point_coincident){
		if(pt2->invisible){
			pt2->invisible=false;
		}
		DeletePtFromCn(&(cns),pt);
		DeletePoint(pt);
		*(lastpt)=NULL;
		return true;
	}
	return false;
}

bool MyCanvas::CheckCoincidency(GLine*ln)
{
	// check coincidency of lines
	bool line_coincident=false;
	GLine *ln1,*ln2=0;
	for(ln1=lns;ln1!=NULL && line_coincident==false;ln1=ln1->next){
		line_coincident=line_coincide_global(cns,ln1,ln);
		ln2=ln1;
	}
	if(line_coincident){
		if(ln2->invisible){
			ln2->invisible=false;
		}
		DeleteLnFromCn(&(cns),ln);
		DeleteLine(ln);
		*(lastln)=NULL;
		return true;
	}
	return false;
}

bool MyCanvas::CheckCoincidency(GCircle*ci)
{
	// check coincidency of lines
	bool circle_coincident=false;
	GCircle* ci1,*ci2=0;
	for(ci1=cis;ci1!=NULL && circle_coincident==false;ci1=ci1->next){
		circle_coincident=circle_coincide_global(cns,ci1,ci);
		ci2=ci1;
	}
	if(circle_coincident){
		if(ci2->invisible){
			ci2->invisible=false;
		}
		DeleteCiFromCn(&(cns),ci);
		DeleteCircle(ci);
		*(lastci)=NULL;
		return true;
	}
	return false;
}

// incident.h
void MyCanvas::ResetLineIcPara(GLine* ln)
{
	GConst *cnp;
	GPoint *ptp1,*ptp2;
	CComplex gpara;
	double para;
	GIncident* ic;
	double g1x,g1y,g2x,g2y,g3x,g3y,g4x,g4y,norm12,norm32,norm4;
	GLine* lnp1,*lnp2;
	CComplex c1x,c1y,c1z,c2x,c2y,c2z,ss;
	FindLnFromCn(cns,ln,&cnp);
	switch(cnp->Const_Type){
	case CONST_JOIN:
		if(ln->NOics<=2){
			return ;
		}
		ptp1=ln->ics->_icpt;
		ptp2=ln->ics->_icnext->_icpt;
		for(ic=ln->ics->_icnext->_icnext;ic!=NULL;ic=ic->_icnext){
			if(ptp1->xz()==ptp2->xz()){
				gpara=(ic->_icpt->yz() - ptp1->yz())/ (ptp2->yz() - ptp1->yz());
			}
			else {
				gpara=(ic->_icpt->xz() - ptp1->xz())/ (ptp2->xz() - ptp1->xz());
			}
			ic->_icpara=gpara;
		}
		break;
	case CONST_PARALLEL: case CONST_PERP:
		if(ln->NOics<=1){
			return ;
		}
		ptp1=ln->ics->_icpt;
		for(ic=ln->ics->_icnext;ic!=NULL;ic=ic->_icnext){
			if(ln->yy == CComplex(0.0,0.0)){
				gpara = (ic->_icpt->yz() - ptp1->yz()) / ln->xx ;
			}
			else {
				gpara = (ic->_icpt->xz() - ptp1->xz()) / ln->yy * CComplex(-1.0,0.0);
			}
			ic->_icpara=gpara;
		}
		break;
	case CONST_3P_BISECTOR:
		if(ln->NOics<=1){
			return ;
		}
		g1x=cnp->PInput1->xz().real();
		g1y=cnp->PInput1->yz().real();
		g2x=cnp->PInput2->xz().real();
		g2y=cnp->PInput2->yz().real();
		g3x=cnp->PInput3->xz().real();
		g3y=cnp->PInput3->yz().real();
		norm12 = sqrt((g1x-g2x)*(g1x-g2x)+(g1y-g2y)*(g1y-g2y));
		norm32 = sqrt((g3x-g2x)*(g3x-g2x)+(g3y-g2y)*(g3y-g2y));
		if(norm12<0.0001){
			norm12=1.0;
			g1x=g2x+1.0;
			g1y=g2y;
		}
		if(norm32<0.0001){
			norm32=1.0;
			g3x=g2x;
			g3y=g2y+1.0;
		}
		g1x=g2x + (g1x-g2x)/norm12;
		g1y=g2y + (g1y-g2y)/norm12;
		g3x=g2x + (g3x-g2x)/norm32;
		g3y=g2y + (g3y-g2y)/norm32;
		g4x = (g1x+g3x)*0.5 - g2x;
		g4y = (g1y+g3y)*0.5 - g2y;
		norm4 = sqrt(g4x*g4x+g4y*g4y);
		if(norm4!=0){
			g4x/=norm4;
			g4y/=norm4;
		}
		else {
			g4x = g1y-g3y;
			g4y = - g1x+g3x;
			norm4 = sqrt(g4x*g4x+g4y*g4y);
			g4x/=norm4;
			g4y/=norm4;
		}
		for(ic=ln->ics->_icnext;ic!=NULL;ic=ic->_icnext){
			if(g4y==0.0){
				para = (ic->_icpt->xz().real() - g2x) / g4x;
			}
			else {
				para = (ic->_icpt->yz().real() - g2y) / g4y;
			}
			ic->_icpara=CComplex(para,0.0);
		}
		break;
	case CONST_BISECTOR:
		lnp1=cnp->LInput1;
		lnp2=cnp->LInput2;
		c1x = lnp1->yy * lnp2->zz -  lnp1->zz * lnp2->yy;
		c1y = lnp1->zz * lnp2->xx -  lnp1->xx * lnp2->zz;
		c1z = lnp1->xx * lnp2->yy -  lnp1->yy * lnp2->xx;
		c2x = sqrtC(lnp2->xx * lnp2->xx + lnp2->yy * lnp2->yy) * lnp1->xx + cnp->para1 * sqrtC(lnp1->xx * lnp1->xx + lnp1->yy * lnp1->yy) * lnp2->xx;
		c2y = sqrtC(lnp2->xx * lnp2->xx + lnp2->yy * lnp2->yy) * lnp1->yy + cnp->para1 * sqrtC(lnp1->xx * lnp1->xx + lnp1->yy * lnp1->yy) * lnp2->yy;
		c2z = sqrtC(lnp2->xx * lnp2->xx + lnp2->yy * lnp2->yy) * lnp1->zz + cnp->para1 * sqrtC(lnp1->xx * lnp1->xx + lnp1->yy * lnp1->yy) * lnp2->zz;
		for(ic=ln->ics;ic!=NULL;ic=ic->_icnext){
			if(c1z==CComplex(0.0,0.0)){
				ic->_icpara=CComplex(0.0,0.0);
			}
			else if(c2y!=CComplex(0.0,0.0)){
				ic->_icpara = ( ic->_icpt->xz() - c1x / c1z) * c2z / c2y ;
			}
			else {
				ic->_icpara = (c1y / c1z - ic->_icpt->yz() ) * c2z / c2x ;
			}
		}
		break;
	case CONST_TANGENT:
		c2x=(cnp->CInput1->yy - cnp->PInput1->yz()) * CComplex(-1.0,0.0) ;
		c2y=(cnp->CInput1->xx - cnp->PInput1->xz());
		for(ic=ln->ics;ic!=NULL;ic=ic->_icnext){
			c1x=cnp->PInput1->xz() - ic->_icpt->xz();
			c1y=cnp->PInput1->yz() - ic->_icpt->yz();
			ss=(c1x*c2x+c1y*c2y)/(c2x*c2x+c2y*c2y);
			ic->_icpara=ss;
		}
		break;
	case CONST_CURVE_TANGENT:
		c2x=cnp->LOutput->yy * CComplex(-1.0,0.0) ;
		c2y=cnp->LOutput->xx ;
		for(ic=ln->ics;ic!=NULL;ic=ic->_icnext){
			c1x=cnp->PInput1->xz() - ic->_icpt->xz();
			c1y=cnp->PInput1->yz() - ic->_icpt->yz();
			ss=(c1x*c2x+c1y*c2y)/(c2x*c2x+c2y*c2y);
			ic->_icpara=ss;
		}
		break;
	case CONST_LINE_WITH_ANGLE:
		if(ln->NOics<=1){
			return ;
		}
		ptp1=ln->ics->_icpt;
		for(ic=ln->ics->_icnext;ic!=NULL;ic=ic->_icnext){
			double theta=cnp->AngleInput1->m_angle.real();
			// direction vector(dx,dy)
			CComplex dx=cnp->LInput1->yy*CComplex(cos(theta),0.0)+cnp->LInput1->xx*CComplex(sin(theta),0.0);
			CComplex dy=cnp->LInput1->yy*CComplex(sin(theta),0.0)-cnp->LInput1->xx*CComplex(cos(theta),0.0);
			if(dx == CComplex(0.0,0.0)){
				gpara = (ic->_icpt->yz() - ptp1->yz()) / dy;
			}
			else {
				gpara = (ic->_icpt->xz() - ptp1->xz()) / dx;
			}
			ic->_icpara=gpara;
		}
		break;
	}
}

void MyCanvas::ResetCircleIcPara(GCircle* ci)
{
	double para;
	GIncident* icp;
	if(ci==NULL){
		return;
	}
	for(icp=ci->ics;icp!=NULL;icp=icp->_icnext){
		para = atan2(- icp->_icpt->yz().real() + ci->yy.real(),
				icp->_icpt->xz().real() - ci->xx.real());
		icp->_icpara=CComplex(para,0.0);
	}
}

